@@ -23,55 +23,48 @@ import org.slf4j.spi.LocationAwareLogger;
 /**
  * <p>
  * This class is a minimal implementation of the original
- * <code>org.apache.log4j.Logger</code> class by delegation of all calls 
- * to a {@link org.slf4j.Logger.Logger} instance.
+ * <code>org.apache.log4j.Category</code> class (as found in log4j 1.2) by
+ * delegation of all calls to a {@link org.slf4j.Logger} instance.
  * </p>
  * 
  * <p>
- * Log4j's <code>trace</code>, <code>debug()</code>, <code>info()</code>, 
- * <code>warn()</code>, <code>error()</code> printing methods are directly 
- * mapped to their SLF4J equivalents. Log4j's <code>fatal()</code> 
- * printing method is mapped to SLF4J's <code>error()</code> method 
- * with a FATAL marker.
+ * Log4j's <code>trace</code>, <code>debug()</code>, <code>info()</code>,
+ * <code>warn()</code>, <code>error()</code> printing methods are directly
+ * mapped to their SLF4J equivalents. Log4j's <code>fatal()</code> printing
+ * method is mapped to SLF4J's <code>error()</code> method with a FATAL marker.
  * 
  * @author S&eacute;bastien Pennec
  * @author Ceki G&uuml;lc&uuml;
  */
-
 public class Category {
 
+  private static final String CATEGORY_FQCN = Category.class.getName();
+
   private String name;
 
-  private org.slf4j.Logger slf4jLogger;
+  protected org.slf4j.Logger slf4jLogger;
   private org.slf4j.spi.LocationAwareLogger locationAwareLogger;
+
+  
   
   private static Marker FATAL_MARKER = MarkerFactory.getMarker("FATAL");
 
   Category(String name) {
     this.name = name;
     slf4jLogger = LoggerFactory.getLogger(name);
-    if(slf4jLogger instanceof LocationAwareLogger) {
+    if (slf4jLogger instanceof LocationAwareLogger) {
       locationAwareLogger = (LocationAwareLogger) slf4jLogger;
     }
   }
 
-  public static Logger getLogger(String name) {
+  public static Category getInstance(Class clazz) {
+    return Log4jLoggerFactory.getLogger(clazz.getName());
+  }
+
+  public static Category getInstance(String name) {
     return Log4jLoggerFactory.getLogger(name);
   }
 
-  public static Logger getLogger(Class clazz) {
-    return getLogger(clazz.getName());
-  }
-
-  /**
-   * Does the obvious.
-   * 
-   * @return
-   */
-  public static Logger getRootLogger() {
-    return getLogger(org.slf4j.Logger.ROOT_LOGGER_NAME);
-  }
-
   /**
    * Returns the obvious.
    * 
@@ -90,24 +83,24 @@ public class Category {
    * @return
    */
   public Level getEffectiveLevel() {
-    if(slf4jLogger.isTraceEnabled()) {
+    if (slf4jLogger.isTraceEnabled()) {
       return Level.TRACE;
     }
-    if(slf4jLogger.isDebugEnabled()) {
+    if (slf4jLogger.isDebugEnabled()) {
       return Level.DEBUG;
     }
-    if(slf4jLogger.isInfoEnabled()) {
+    if (slf4jLogger.isInfoEnabled()) {
       return Level.INFO;
     }
-    if(slf4jLogger.isWarnEnabled()) {
+    if (slf4jLogger.isWarnEnabled()) {
       return Level.WARN;
     }
     return Level.ERROR;
   }
 
   /**
-   * Returns the assigned {@link Level}, if any, for this Category.
-   * This implementation always returns null.
+   * Returns the assigned {@link Level}, if any, for this Category. This
+   * implementation always returns null.
    * 
    * @return Level - the assigned Level, can be <code>null</code>.
    */
@@ -115,70 +108,53 @@ public class Category {
     return null;
   }
 
-
   /**
    * @deprecated Please use {@link #getLevel} instead.
-  */
+   */
   final public Level getPriority() {
     return null;
   }
-  
-  /**
-   * Delegates to {@link org.slf4j.Logger#isTraceEnabled} 
-   * method of SLF4J.
-   */
-  public boolean isTraceEnabled() {
-    return slf4jLogger.isTraceEnabled();
-  }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#isDebugEnabled} method in  SLF4J
+   * Delegates to {@link org.slf4j.Logger#isDebugEnabled} method in SLF4J
    */
   public boolean isDebugEnabled() {
     return slf4jLogger.isDebugEnabled();
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#isInfoEnabled} method in  SLF4J
+   * Delegates to {@link org.slf4j.Logger#isInfoEnabled} method in SLF4J
    */
   public boolean isInfoEnabled() {
     return slf4jLogger.isInfoEnabled();
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#isWarnEnabled} method in  SLF4J
+   * Delegates tob {@link org.slf4j.Logger#isWarnEnabled} method in SLF4J
    */
   public boolean isWarnEnabled() {
     return slf4jLogger.isWarnEnabled();
   }
-  
+
   /**
    * Delegates to {@link org.slf4j.Logger#isErrorEnabled} method in SLF4J
    */
   public boolean isErrorEnabled() {
     return slf4jLogger.isErrorEnabled();
   }
-  
-  /**
-   * Delegates to {@link #isEnabledFor(Level)}.
-   * 
-   * @param p the priority to check against
-   * @return true if this logger is enabled for the given priority, false otehrwise.
-   */
-  public boolean isEnabledFor(Priority p) {
-    return isEnabledFor(Level.toLevel(p.level));
-  }
 
   /**
-   * Determines whether the level passes as parameter is enabled in
-   * the underlying SLF4J logger. Each log4j level is mapped directly to
-   * its SLF4J equivalent, except for FATAL which is mapped as ERROR. 
+   * Determines whether the priority passed as parameter is enabled in the
+   * underlying SLF4J logger. Each log4j priority is mapped directly to its
+   * SLF4J equivalent, except for FATAL which is mapped as ERROR.
    * 
-   * @param l the level to check against
-   * @return true if this logger is enabled for the given level, false otehrwise.
+   * @param p
+   *          the priority to check against
+   * @return true if this logger is enabled for the given level, false
+   *         otherwise.
    */
-  public boolean isEnabledFor(Level l) {
-    switch (l.level) {
+  public boolean isEnabledFor(Priority p) {
+    switch (p.level) {
     case Level.TRACE_INT:
       return slf4jLogger.isTraceEnabled();
     case Level.DEBUG_INT:
@@ -195,120 +171,133 @@ public class Category {
     return false;
   }
 
-  /**
-   * Delegates to {@link org.slf4j.Logger#trace(String)} method in SLF4J.
-   */
-  public void trace(Object message) {
-    // casting to String as SLF4J only accepts String instances, not Object
-    // instances.
-    slf4jLogger.trace(convertToString(message));
+  void differentiatedLog(Marker marker, String fqcn, int level, Object message,
+      Throwable t) {
+    String m = convertToString(message);
+    if (locationAwareLogger != null) {
+      locationAwareLogger.log(marker, fqcn, level, m, t);
+    } else {
+      switch (level) {
+      case LocationAwareLogger.TRACE_INT:
+        slf4jLogger.trace(marker, m);
+        break;
+      case LocationAwareLogger.DEBUG_INT:
+        slf4jLogger.debug(marker, m);
+        break;
+      case LocationAwareLogger.INFO_INT:
+        slf4jLogger.info(marker, m);
+        break;
+      case LocationAwareLogger.WARN_INT:
+        slf4jLogger.warn(marker, m);
+        break;
+      case LocationAwareLogger.ERROR_INT:
+        slf4jLogger.error(marker, m);
+        break;
+      }
+    }
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#trace(String,Throwable)} 
-   * method in SLF4J.
-   */
-  public void trace(Object message, Throwable t) {
-    slf4jLogger.trace(convertToString(message), t);
-  }
-  
-  /**
-   * Delegates to {@link org.slf4j.Logger#debug(String)} method of
-   * SLF4J.
+   * Delegates to {@link org.slf4j.Logger#debug(String)} method of SLF4J.
    */
   public void debug(Object message) {
-    // casting to String as SLF4J only accepts String instances, not Object
-    // instances.
-    slf4jLogger.debug(convertToString(message));
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.DEBUG_INT,
+        message, null);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#debug(String,Throwable)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#debug(String,Throwable)} method in
+   * SLF4J.
    */
   public void debug(Object message, Throwable t) {
-    slf4jLogger.debug(convertToString(message), t);
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.DEBUG_INT,
+        message, t);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#info(String)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#info(String)} method in SLF4J.
    */
   public void info(Object message) {
-    slf4jLogger.info(convertToString(message));
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.INFO_INT,
+        message, null);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#info(String,Throwable)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#info(String,Throwable)} method in
+   * SLF4J.
    */
   public void info(Object message, Throwable t) {
-    slf4jLogger.info(convertToString(message), t);
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.INFO_INT,
+        message, t);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#warn(String)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#warn(String)} method in SLF4J.
    */
   public void warn(Object message) {
-    slf4jLogger.warn(convertToString(message));
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.WARN_INT,
+        message, null);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#warn(String,Throwable)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#warn(String,Throwable)} method in
+   * SLF4J.
    */
   public void warn(Object message, Throwable t) {
-    slf4jLogger.warn(convertToString(message), t);
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.WARN_INT,
+        message, t);
   }
 
-  
   /**
-   * Delegates to {@link org.slf4j.Logger#error(String)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#error(String)} method in SLF4J.
    */
   public void error(Object message) {
-    slf4jLogger.error(convertToString(message));
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.ERROR_INT,
+        message, null);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#error(String,Throwable)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#error(String,Throwable)} method in
+   * SLF4J.
    */
   public void error(Object message, Throwable t) {
-    slf4jLogger.error(convertToString(message), t);
+    differentiatedLog(null, CATEGORY_FQCN, LocationAwareLogger.ERROR_INT,
+        message, t);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#error(String)} 
-   * method in SLF4J.
+   * Delegates to {@link org.slf4j.Logger#error(String)} method in SLF4J.
    */
   public void fatal(Object message) {
-    slf4jLogger.error(FATAL_MARKER, convertToString(message));
+    differentiatedLog(FATAL_MARKER, CATEGORY_FQCN,
+        LocationAwareLogger.ERROR_INT, message, null);
   }
 
   /**
-   * Delegates to {@link org.slf4j.Logger#error(String,Throwable)} 
-   * method in SLF4J. In addition, the call is marked with a marker named "FATAL".
+   * Delegates to {@link org.slf4j.Logger#error(String,Throwable)} method in
+   * SLF4J. In addition, the call is marked with a marker named "FATAL".
    */
   public void fatal(Object message, Throwable t) {
-    slf4jLogger.error(FATAL_MARKER, convertToString(message), t);
+    differentiatedLog(FATAL_MARKER, CATEGORY_FQCN,
+        LocationAwareLogger.ERROR_INT, message, t);
   }
 
+  // See also http://bugzilla.slf4j.org/show_bug.cgi?id=168
   public void log(String FQCN, Priority p, Object msg, Throwable t) {
     int levelInt = priorityToLevelInt(p);
-    if(locationAwareLogger != null) {
-      if(msg != null) {
-        locationAwareLogger.log(null, FQCN, levelInt, msg.toString(), t); 
-      } else {
-        locationAwareLogger.log(null, FQCN, levelInt, null, t); 
-      }
-    } else {
-      throw new UnsupportedOperationException("The logger ["+slf4jLogger+"] does not seem to be location aware.");
-    }
-   
+    differentiatedLog(null, FQCN, levelInt, msg, t);
   }
-  
+
+  public void log(Priority p, Object message, Throwable t) {
+    int levelInt = priorityToLevelInt(p);
+    differentiatedLog(null, CATEGORY_FQCN, levelInt, message, t);
+  }
+
+  public void log(Priority p, Object message) {
+    int levelInt = priorityToLevelInt(p);
+    differentiatedLog(null, CATEGORY_FQCN, levelInt, message, null);
+  }
+
   private int priorityToLevelInt(Priority p) {
     switch (p.level) {
     case Level.TRACE_INT:
@@ -327,10 +316,10 @@ public class Category {
       throw new IllegalStateException("Unknown Priority " + p);
     }
   }
-  
-  private final String convertToString(Object message) {
+
+  protected final String convertToString(Object message) {
     if (message == null) {
-      return (String)message;
+      return (String) message;
     } else {
       return message.toString();
     }
