@@ -32,6 +32,8 @@
 
 package org.slf4j;
 
+import java.io.PrintStream;
+
 import junit.framework.TestCase;
 
 
@@ -43,16 +45,20 @@ import junit.framework.TestCase;
  */
 public class InvocationTest extends TestCase {
 
+  PrintStream old = System.err;
+  
   public InvocationTest (String arg0) {
     super(arg0);
   }
 
   protected void setUp() throws Exception {
     super.setUp();
+    System.setErr(new SilentPrintStream(old));
   }
 
   protected void tearDown() throws Exception {
     super.tearDown();
+    System.setErr(old);
   }
   
   public void test1() {
